@@ -2,7 +2,9 @@ import numpy as np
 import torch
 import itertools
 import sys
+import random
 from .utils_fa import covariance
+from .utils_fa import set_seed
 from typing import Union, Tuple, List, Optional, Callable
 from tqdm import tqdm
 from torch import nn, optim
@@ -10,8 +12,9 @@ import warnings
 
 warnings.filterwarnings("ignore")
 
+
 class Model_sjSDM:
-    def __init__(self, device: str = "cpu", dtype: str = "float32"):
+    def __init__(self, device: str = "cpu", dtype: str = "float32", seed: int = 42):
         """sjSDM constructor
 
         Args:
@@ -19,6 +22,10 @@ class Model_sjSDM:
             dtype (str, optional): Dtype. Defaults to "float32".
 
         """
+        
+        
+        self.seed = seed
+        set_seed(seed)
         self.params = []
         self.losses = []
         self.env = None
@@ -333,11 +340,11 @@ class Model_sjSDM:
 
         """              
         
-        if self.device.type == 'cuda' and torch.cuda.is_available():
+        if (self.device.type == 'cuda' and torch.cuda.is_available()) or self.device.type == "mps":
             #torch.set_default_tensor_type('torch.cuda.FloatTensor')
-            self.env.cuda(self.device) # type: ignore
+            self.env.to(self.device) # type: ignore
             if self.spatial is not None:
-                self.spatial.cuda(self.device) # type: ignore
+                self.spatial.to(self.device) # type: ignore
         else:
             torch.set_default_tensor_type('torch.FloatTensor')
         
@@ -383,7 +390,8 @@ class Model_sjSDM:
             epochs: int = 100, 
             sampling: int = 100, 
             parallel: int = 0,
-            early_stopping_training: int = -1) -> None:
+            early_stopping_training: int = -1,
+            verbose: bool = True) -> None:
         """Fit sjSDM model
 
         Args:
@@ -405,7 +413,7 @@ class Model_sjSDM:
         df = self.df
         alpha = self.alpha
         
-        if self.device.type == 'cuda': # type: ignore
+        if self.device.type == 'cuda' or self.device.type == 'mps': # type: ignore
             device = self.device.type+ ":" + str(self.device.index) # type: ignore
         else:
             device = 'cpu'
@@ -432,7 +440,11 @@ class Model_sjSDM:
                 self.optimizer.step()
 
         desc='loss: Inf'
-        ep_bar = tqdm(range(epochs),bar_format= "Iter: {n_fmt}/{total_fmt} {l_bar}{bar}| [{elapsed}, {rate_fmt}{postfix}]", file=sys.stdout)
+        if verbose:
+            ep_bar = tqdm(range(epochs),bar_format= "Iter: {n_fmt}/{total_fmt} {l_bar}{bar}| [{elapsed}, {rate_fmt}{postfix}]", file=sys.stdout)
+        else:
+            ep_bar = range(epochs)
+            
         if type(SP) is np.ndarray:
             for epoch in ep_bar:
                 for step, (x, y, sp) in enumerate(dataLoader):
@@ -452,7 +464,8 @@ class Model_sjSDM:
                     batch_loss[step] = loss.item()
                 bl = np.mean(batch_loss)
                 bl = np.round(bl, 3)
-                ep_bar.set_postfix(loss=f'{bl}')
+                if verbose:
+                    ep_bar.set_postfix(loss=f'{bl}')
 
                 self.history[epoch] = bl
                 if self.useSched:
@@ -485,7 +498,8 @@ class Model_sjSDM:
                     batch_loss[step] = loss.item()
                 bl = np.mean(batch_loss)
                 bl = np.round(bl, 3)
-                ep_bar.set_postfix(loss=f'{bl}')
+                if verbose:
+                    ep_bar.set_postfix(loss=f'{bl}')
                 self.history[epoch] = bl
                 if self.useSched:
                     self.scheduler.step(bl)
@@ -532,7 +546,7 @@ class Model_sjSDM:
         torch.cuda.empty_cache()
         any_losses = len(self.losses) > 0
 
-        if self.device.type == 'cuda': # type: ignore
+        if self.device.type == 'cuda' or self.device.type == "mps": # type: ignore
             device = self.device.type+ ":" + str(self.device.index) # type: ignore
         else:
             device = 'cpu'
@@ -598,7 +612,7 @@ class Model_sjSDM:
         loss_function = self._build_loss_function(train = False, raw=not link, simulate=simulate)
 
         pred = []
-        if self.device.type == 'cuda': # type: ignore
+        if self.device.type == 'cuda' or self.device.type == "mps": # type: ignore
             device = self.device.type+ ":" + str(self.device.index) # type: ignore
         else:
             device = 'cpu'
@@ -642,7 +656,8 @@ class Model_sjSDM:
            SP: Optional[np.ndarray] = None, 
            batch_size: int = 25, 
            parallel: int = 0, 
-           sampling: int = 100) -> List[np.ndarray]:
+           sampling: int = 100,
+           verbose: bool = True) -> List[np.ndarray]:
         """Calculate standard errors for environmental coefficients
 
         Args:
@@ -662,7 +677,7 @@ class Model_sjSDM:
         weights_base = np.transpose(self.env_weights[0])
         y_dim = Y.shape[1]
 
-        if self.device.type == 'cuda':
+        if self.device.type == 'cuda' or self.device.type == "mps":
             device = self.device.type+ ":" + str(self.device.index)
         else:
             device = 'cpu'
@@ -670,8 +685,11 @@ class Model_sjSDM:
         _ = sys.stdout.write("\nCalculating standard errors...\n")
 
         desc='loss: Inf'
-        sp_bar = tqdm(range(Y.shape[1]),bar_format= "Species: {n_fmt}/{total_fmt} {l_bar}{bar}| [{elapsed}, {rate_fmt}]", file=sys.stdout)
-
+        if verbose:
+            sp_bar = tqdm(range(Y.shape[1]),bar_format= "Species: {n_fmt}/{total_fmt} {l_bar}{bar}| [{elapsed}, {rate_fmt}]", file=sys.stdout)
+        else:
+            sp_bar = range(Y.shape[1])
+        
         if type(SP) is np.ndarray:
             for i in sp_bar:
                 weights = torch.tensor(weights_base[:,i].reshape([-1,1]), device=self.device, dtype=self.dtype, requires_grad=True).to(self.device)
@@ -700,7 +718,7 @@ class Model_sjSDM:
                         hessian_out = hessian
                     else:
                         hessian_out += hessian
-                se.append(torch.sqrt(torch.diag(torch.inverse(hessian_out))).data.cpu().numpy())
+                se.append(torch.sqrt(torch.diag(torch.inverse(hessian_out+0.001))).data.cpu().numpy())
             return se                
         else:
             for i in sp_bar:
@@ -729,7 +747,7 @@ class Model_sjSDM:
                         hessian_out = hessian
                     else:
                         hessian_out += hessian
-                se.append(torch.sqrt(torch.diag(torch.inverse(hessian_out))).data.cpu().numpy())
+                se.append(torch.sqrt(torch.diag(torch.inverse(hessian_out+0.001))).data.cpu().numpy())
             return se
 
     def _build_cov_constrain_function(self, 
@@ -835,7 +853,7 @@ class Model_sjSDM:
 
         if train:
             if self.link == "logit" or self.link == "probit":
-                @torch.jit.script
+                #@torch.jit.script
                 def tmp(mu: torch.Tensor, Ys: torch.Tensor, sigma: torch.Tensor, batch_size: int, sampling: int, df: int, alpha: float, device: str, dtype: torch.dtype):
                     noise = torch.randn(size = [sampling, batch_size, df], device=torch.device(device), dtype=dtype)
                     E = torch.sigmoid(   torch.einsum("ijk, lk -> ijl", [noise, sigma]).add(mu).mul(alpha)   ).mul(0.999999).add(0.0000005)
@@ -845,7 +863,7 @@ class Model_sjSDM:
                     loss = Eprob.log().neg().sub(maxlogprob)
                     return loss
             elif self.link == "linear":
-                @torch.jit.script
+                #@torch.jit.script
                 def tmp(mu: torch.Tensor, Ys: torch.Tensor, sigma: torch.Tensor, batch_size: int, sampling: int, df: int, alpha: float, device: str, dtype: torch.dtype):
                     noise = torch.randn(size = [sampling, batch_size, df], device=torch.device(device), dtype=dtype)
                     E = torch.clamp(torch.einsum("ijk, lk -> ijl", [noise, sigma]).add(mu).mul(alpha), 0.0, 1.0).mul(0.999999).add(0.0000005)
@@ -876,7 +894,7 @@ class Model_sjSDM:
 
             elif self.link == "normal":
                 def tmp(mu: torch.Tensor, Ys: torch.Tensor, sigma: torch.Tensor, batch_size: int, sampling: int, df: int, alpha: float, device: str, dtype: torch.dtype):
-                    return torch.distributions.MultivariateNormal(loc=mu, covariance_matrix=sigma.matmul(sigma.t()).add(torch.eye(sigma.shape[0]))).log_prob(Ys).neg()
+                    return torch.distributions.MultivariateNormal(loc=mu, covariance_matrix=sigma.matmul(sigma.t()).add(torch.eye(sigma.shape[0], device=torch.device(device), dtype=dtype))).log_prob(Ys).neg()
         else:
             if self.link == "probit": 
                 link_func = lambda value: torch.distributions.Normal(0.0, 1.0).cdf(value)
@@ -910,13 +928,12 @@ class Model_sjSDM:
                     noise = torch.randn(size = [sampling, batch_size, df], device=torch.device(device), dtype=dtype)
                     E = torch.sigmoid(   torch.einsum("ijk, lk -> ijl", [noise, sigma]).add(mu).mul(alpha)   ).mul(0.999999).add(0.0000005)
                     logprob = E.log().mul(Ys).add((1.0 - E).log().mul(1.0 - Ys)).neg()#.sum(dim = 2).neg()
-                    Prop = logprob/logprob.sum(dim = 2).reshape([sampling, batch_size, 1])
                     logprob = logprob.sum(dim = 2).neg()
                     maxlogprob = logprob.max(dim = 0).values
                     Eprob = logprob.sub(maxlogprob).exp().mean(dim = 0)
                     return Eprob.log().neg().sub(maxlogprob).reshape([batch_size, 1])
             elif self.link == "linear":
-                @torch.jit.script
+                #@torch.jit.script
                 def tmp(mu: torch.Tensor, Ys: torch.Tensor, sigma: torch.Tensor, batch_size: int, sampling: int, df: int, alpha: float, device: str, dtype: torch.dtype):
                     noise = torch.randn(size = [sampling, batch_size, df], device=torch.device(device), dtype=dtype)
                     E = torch.clamp(torch.einsum("ijk, lk -> ijl", [noise, sigma]).add(mu).mul(alpha), 0.0, 1.0).mul(0.999999).add(0.0000005)
@@ -948,7 +965,7 @@ class Model_sjSDM:
 
             elif self.link == "normal":
                 def tmp(mu: torch.Tensor, Ys: torch.Tensor, sigma: torch.Tensor, batch_size: int, sampling: int, df: int, alpha: float, device: str, dtype: torch.dtype):
-                    return torch.distributions.MultivariateNormal(loc=mu, covariance_matrix=sigma.matmul(sigma.t()).add(torch.eye(sigma.shape[0]))).log_prob(Ys).neg()
+                    return torch.distributions.MultivariateNormal(loc=mu, covariance_matrix=sigma.matmul(sigma.t()).add(torch.eye(sigma.shape[0]), device=device, dtype=dtype )).log_prob(Ys).neg()
 
         return tmp
 
